; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; Nom du projet : Dev-Module
; Nom du fichier : DevModule - DevModule.pb
; Version du fichier : 0.9.9
; Programmation : En cours
; Programm par : Guillaume Saumure
; Alias : StarBootics
; Courriel : gsaumure@cgocable.ca
; Date : 03-10-2014
; Mise  jour : 01-06-2018
; Code PureBasic : V5.62
; Plateforme : Windows, Linux, MacOS X
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

DeclareModule DevModule
  
  Declare Initialize()
  Declare CreateWindow()
  Declare EventManager()
  
EndDeclareModule

Module DevModule

  IncludeFile "DevModule - Constants.pb"
  IncludeFile "DevModule - Language.pb"
  IncludeFile "DevModule - Extra.pb"
  
  Global PrefFileName.s
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< OpenDevModulePreferences (Private) <<<<<<
  
  Procedure OpenDevModulePreferences()
    
    If OpenPreferences(PrefFileName)
      
      Window::ReadPrefs("MainWin", #MainWin)
      Window::ReadPrefs("InternalStructureWin", #InternalStructureWin)
      Language::ReadPrefs("Language")
      Options::ReadPrefs("Options")
      Sticky::ReadPrefs("Sticky")
      Comments::ReadPrefs("Comments")
      Field::ReadPrefs("CodeGenerator")
      Stats::ReadPrefs("Statistics")
      
      ClosePreferences()
      
    EndIf
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< CreateDevModulePreferences (Private) <<<<<<
  
  Procedure CreateDevModulePreferences()
    
    If CreatePreferences(PrefFileName)
      
      Window::WritePrefs("MainWin", #MainWin)
      Window::WritePrefs("InternalStructureWin", #InternalStructureWin)
      Language::WritePrefs("Language")
      Options::WritePrefs("Options")
      Sticky::WritePrefs("Sticky")
      Comments::WritePrefs("Comments")
      Field::WritePrefs("CodeGenerator")
      Stats::WritePrefs("Statistics")
      
      ClosePreferences()
      
    EndIf
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< ShutDown (Private) <<<<<
  
  Procedure ShutDown()
    
    Window::GetCurrentPositionSize(#MainWin)
    Comments::From_GUI()
    
    CreateDevModulePreferences()
    
    For GadgetID = #CheckBox_Observator To #CheckBox_ReadBinary Step 3
      UnbindGadgetEvent(GadgetID, @MethodGadgetFollowCheckBox())
    Next
    
    UnbindGadgetEvent(#TrackBar_Tabulation_Size, @TextFollowTrackBar()) 
    UnbindGadgetEvent(#Option_Use_NBSPACE_Tabulation, @TrackBarFollowOption())
    
    RemoveKeyboardShortcut(#MainWin, #PB_Shortcut_All)
    
    ColorPreview::Free(#PreviewColor_Editor_FrontColor)
    ColorPreview::Free(#PreviewColor_Editor_BackColor)
    
    CloseWindow(#MainWin)
    
    Language::Reset()
    Window::Reset(-1)
    Sticky::Reset()
    Options::Reset()
    Field::Reset(Field::#RESET_STYLE_TERMINAL)
    MatrixGenerator::Reset()
    Comments::Reset()
    AdvancedDateFormatting::Reset()
    AskPureBasicCompiler::Reset()
    Stats::Reset()
    Chrono::Reset(-1)
    
    PrefFileName = ""
    End
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Nom de la procdure <<<<<
  
  Procedure From_GUI()
    
    If Trim(GetGadgetText(#String_Module_Name)) = ""
      MessageRequester(Language::Message("Dev-Module - Error"), Language::Message("The Module name is missing !"))
      ProcedureReturn #False
    EndIf
    
    If Comments::DateMaskCount() = 0
      MessageRequester(Language::Message("Dev-Module - Error"), Language::Message("No Date Mask !"))
      ProcedureReturn #False
    EndIf
    
    Field::SetModuleName(Trim(GetGadgetText(#String_Module_Name)))
    
    If GetGadgetState(#Option_Single_Instance) = 1
      Field::SetInstanceStyle(Field::#INSTANCE_STYLE_UNIQUE)
    ElseIf GetGadgetState(#Option_Multiple_Instance_ID) = 1
      Field::SetInstanceStyle(Field::#INSTANCE_STYLE_MULTIPLE_ID)
    ElseIf GetGadgetState(#Option_Multiple_Instance_Index) = 1
      Field::SetInstanceStyle(Field::#INSTANCE_STYLE_MULTIPLE_INDEX)
    ElseIf GetGadgetState(#Option_Multiple_Instance_Name) = 1
      Field::SetInstanceStyle(Field::#INSTANCE_STYLE_MULTIPLE_NAME)
    ElseIf GetGadgetState(#Option_Off_Module_Instance) = 1
      Field::SetInstanceStyle(Field::#INSTANCE_STYLE_OFF_MODULE)
    EndIf

    PastedCodeCleanUp::DoIt(#Editor_Module_Primary)
    PastedCodeCleanUp::DoIt(#Editor_Module_Secondary)

    Field_Max = CountGadgetItems(#Editor_Module_Primary) - 1
    
    If Field::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_ID
      
      For FieldID = 0 To Field_Max
        
        If GetGadgetItemText(#Editor_Module_Primary, FieldID , 0) = "ID.l"
          ID_Field_Found = #True
          Break
        EndIf
        
      Next
      
      If ID_Field_Found = #False
        AddGadgetItem(#Editor_Module_Primary, 0, "ID.l")
      EndIf
      
    EndIf
    
    Field_Max = CountGadgetItems(#Editor_Module_Primary) - 1
    
    For FieldID = 0 To Field_Max
      
      Definition.s = GetGadgetItemText(#Editor_Module_Primary, FieldID , 0)
      
      If Definition <> ""
        Field::New(0, FieldID, Definition)
      EndIf
      
    Next
    
    Field_Max = CountGadgetItems(#Editor_Module_Secondary) - 1
    
    If Field_Max >= 0  
      Field::SetSecondaryInstance(#True)
    EndIf
    
    For FieldID = 0 To Field_Max
      
      Definition.s = GetGadgetItemText(#Editor_Module_Secondary, FieldID , 0)
      
      If Definition <> ""
        Field::New(1, FieldID, Definition)
      EndIf
      
    Next

    If GetGadgetState(#CheckBox_Observator)
      If GetGadgetState(#Option_Observator_Public)
        BasicMethods::SetObservator(1)
      ElseIf GetGadgetState(#Option_Observator_Private)
        BasicMethods::SetObservator(2)
      EndIf
    Else
      BasicMethods::SetObservator(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Mutator)
      If GetGadgetState(#Option_Mutator_Public)
        BasicMethods::SetMutator(1)
      ElseIf GetGadgetState(#Option_Mutator_Private)
        BasicMethods::SetMutator(2)
      EndIf
    Else
      BasicMethods::SetMutator(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Update)
      If GetGadgetState(#Option_Update_Public)
        BasicMethods::SetUpdate(1)
      ElseIf GetGadgetState(#Option_Update_Private)
        BasicMethods::SetUpdate(2)
      EndIf
    Else
      BasicMethods::SetUpdate(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Initialize)
      If GetGadgetState(#Option_Initialize_Public)
        BasicMethods::SetInitialize(1)
      ElseIf GetGadgetState(#Option_Initialize_Private)
        BasicMethods::SetInitialize(2)
      EndIf
    Else
      BasicMethods::SetInitialize(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Reset)
      If GetGadgetState(#Option_Reset_Public)
        BasicMethods::SetReset(1)
      ElseIf GetGadgetState(#Option_Reset_Private)
        BasicMethods::SetReset(2)
      EndIf
    Else
      BasicMethods::SetReset(0)
    EndIf
    
    If GetGadgetState(#CheckBox_QuickSortAscending)
      If GetGadgetState(#Option_QuickSortAscending_Public)
        StaticArrayMethods::SetQuickSortAscendant(1)
      ElseIf GetGadgetState(#Option_QuickSortAscending_Private)
        StaticArrayMethods::SetQuickSortAscendant(2)
      EndIf
    Else
      StaticArrayMethods::SetQuickSortAscendant(0)
    EndIf
    
    If GetGadgetState(#CheckBox_QuickSortDescending)
      If GetGadgetState(#Option_QuickSortDescending_Public)
        StaticArrayMethods::SetQuickSortDescendant(1)
      ElseIf GetGadgetState(#Option_QuickSortDescending_Private)
        StaticArrayMethods::SetQuickSortDescendant(2)
      EndIf
    Else
      StaticArrayMethods::SetQuickSortDescendant(0)
    EndIf
    
    If GetGadgetState(#CheckBox_LinearSearch)
      If GetGadgetState(#Option_LinearSearch_Public)
        StaticArrayMethods::SetLinearSearch(1)
      ElseIf GetGadgetState(#Option_LinearSearch_Private)
        StaticArrayMethods::SetLinearSearch(2)
      EndIf
    Else
      StaticArrayMethods::SetLinearSearch(0)
    EndIf
    
    If GetGadgetState(#CheckBox_BinarySearch)
      If GetGadgetState(#Option_BinarySearch_Public)
        StaticArrayMethods::SetBinarySearch(1)
      ElseIf GetGadgetState(#Option_BinarySearch_Private)
        StaticArrayMethods::SetBinarySearch(2)
      EndIf
    Else
      StaticArrayMethods::SetBinarySearch(0)
    EndIf  
    
    If GetGadgetState(#CheckBox_FixedLengthQueue)
      If GetGadgetState(#Option_FixedLengthQueue_Public)
        StaticArrayMethods::SetFixedLenghtQueue(1)
      ElseIf GetGadgetState(#Option_FixedLengthQueue_Private)
        StaticArrayMethods::SetFixedLenghtQueue(2)
      EndIf
    Else
      StaticArrayMethods::SetFixedLenghtQueue(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Sort_Linked_List)
      If GetGadgetState(#Option_Sort_Linked_List_Public)
        LinkedListMethods::SetSort(1)
      ElseIf GetGadgetState(#Option_Sort_Linked_List_Private)
        LinkedListMethods::SetSort(2)
      EndIf
    Else
      LinkedListMethods::SetSort(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Lin_Search_Linked_List)
      If GetGadgetState(#Option_Lin_Search_Linked_List_Public)
        LinkedListMethods::SetLinearSearch(1)
      ElseIf GetGadgetState(#Option_Lin_Search_Linked_List_Private)
        LinkedListMethods::SetLinearSearch(2)
      EndIf
    Else
      LinkedListMethods::SetLinearSearch(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Bin_Search_Linked_List)
      If GetGadgetState(#Option_Bin_Search_Linked_List_Public)
        LinkedListMethods::SetBinarySearch(1)
      ElseIf GetGadgetState(#Option_Bin_Search_Linked_List_Private)
        LinkedListMethods::SetBinarySearch(2)
      EndIf
    Else
      LinkedListMethods::SetBinarySearch(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Sort_Dynamic_Array)
      If GetGadgetState(#Option_Sort_Dynamic_Array_Public)
        DynamicArrayMethods::SetSort(1)
      ElseIf GetGadgetState(#Option_Sort_Dynamic_Array_Private)
        DynamicArrayMethods::SetSort(2)
      EndIf
    Else
      DynamicArrayMethods::SetSort(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Lin_Search_Dynamic_Array)
      If GetGadgetState(#Option_Lin_Search_Dynamic_Array_Public)
        DynamicArrayMethods::SetLinearSearch(1)
      ElseIf GetGadgetState(#Option_Lin_Search_Dynamic_Array_Private)
        DynamicArrayMethods::SetLinearSearch(2)
      EndIf
    Else
      DynamicArrayMethods::SetLinearSearch(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Bin_Search_Dynamic_Array)
      If GetGadgetState(#Option_Bin_Search_Dynamic_Array_Public)
        DynamicArrayMethods::SetBinarySearch(1)
      ElseIf GetGadgetState(#Option_Bin_Search_Dynamic_Array_Private)
        DynamicArrayMethods::SetBinarySearch(2)
      EndIf
    Else
      DynamicArrayMethods::SetBinarySearch(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Equal)
      If GetGadgetState(#Option_Equal_Public)
        MathsMethods::SetEqual(1)
      ElseIf GetGadgetState(#Option_Equal_Private)
        MathsMethods::SetEqual(2)
      EndIf
    Else
      MathsMethods::SetEqual(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Plus)
      If GetGadgetState(#Option_Plus_Public)
        MathsMethods::SetPlus(1)
      ElseIf GetGadgetState(#Option_Plus_Private)
        MathsMethods::SetPlus(2)
      EndIf
    Else
      MathsMethods::SetPlus(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Minus)
      If GetGadgetState(#Option_Minus_Public)
        MathsMethods::SetMinus(1)
      ElseIf GetGadgetState(#Option_Minus_Private)
        MathsMethods::SetMinus(2)
      EndIf
    Else
      MathsMethods::SetMinus(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Product)
      If GetGadgetState(#Option_Product_Public)
        MathsMethods::SetProduct(1)
      ElseIf GetGadgetState(#Option_Product_Private)
        MathsMethods::SetProduct(2)
      EndIf
    Else
      MathsMethods::SetProduct(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Divide)
      If GetGadgetState(#Option_Divide_Public)
        MathsMethods::SetDivide(1)
      ElseIf GetGadgetState(#Option_Divide_Private)
        MathsMethods::SetDivide(2)
      EndIf
    Else
      MathsMethods::SetDivide(0)
    EndIf

    If GetGadgetState(#CheckBox_ProductByScalar)
      If GetGadgetState(#Option_ProductByScalar_Public)
        MathsMethods::SetProductByScalar(1)
      ElseIf GetGadgetState(#Option_ProductByScalar_Private)
        MathsMethods::SetProductByScalar(2)
      EndIf
    Else
      MathsMethods::SetProductByScalar(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DivideByScalar)
      If GetGadgetState(#Option_DivideByScalar_Public)
        MathsMethods::SetDivideByScalar(1)
      ElseIf GetGadgetState(#Option_DivideByScalar_Private)
        MathsMethods::SetDivideByScalar(2)
      EndIf
    Else
      MathsMethods::SetDivideByScalar(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Copy)
      If GetGadgetState(#Option_Copy_Public)
        MathsMethods::SetCopy(1)
      ElseIf GetGadgetState(#Option_Copy_Private)
        MathsMethods::SetCopy(2)
      EndIf
    Else
      MathsMethods::SetCopy(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Compare)
      If GetGadgetState(#Option_Compare_Public)
        MathsMethods::SetCompare(1)
      ElseIf GetGadgetState(#Option_Compare_Private)
        MathsMethods::SetCompare(2)
      EndIf
    Else
      MathsMethods::SetCompare(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Swapping)
      If GetGadgetState(#Option_Swapping_Public)
        MathsMethods::SetSwapping(1)
      ElseIf GetGadgetState(#Option_Swapping_Private)
        MathsMethods::SetSwapping(2)
      EndIf
    Else
      MathsMethods::SetSwapping(0)
    EndIf
    
    If GetGadgetState(#CheckBox_XML_SetGet_Node)
      If GetGadgetState(#Option_XML_SetGet_Node_Public)
        XMLFileMethods::SetNodeWithChildNode(1)
      ElseIf GetGadgetState(#Option_XML_SetGet_Node_Private)
        XMLFileMethods::SetNodeWithChildNode(2)
      EndIf
    Else
      XMLFileMethods::SetNodeWithChildNode(0)
    EndIf
    
    If GetGadgetState(#CheckBox_XML_SetGet_Attribute)
      If GetGadgetState(#Option_XML_SetGet_Attribute_Public)
        XMLFileMethods::SetNodeWithAttributes(1)
      ElseIf GetGadgetState(#Option_XML_SetGet_Attribute_Private)
        XMLFileMethods::SetNodeWithAttributes(2)
      EndIf
    Else
      XMLFileMethods::SetNodeWithAttributes(0)
    EndIf
    
    If GetGadgetState(#CheckBox_XML_AddCollect_Attribute)
      If GetGadgetState(#Option_XML_AddCollect_Attribute_Public)
        XMLFileMethods::SetAddCollectAttributes(1)
      ElseIf GetGadgetState(#Option_XML_AddCollect_Attribute_Private)
        XMLFileMethods::SetAddCollectAttributes(2)
      EndIf
    Else
      XMLFileMethods::SetAddCollectAttributes(0)
    EndIf
    
    If GetGadgetState(#CheckBox_XML_CreateLoad_File)
      If GetGadgetState(#Option_XML_CreateLoad_File_Public)
        XMLFileMethods::SetCreateLoadFile(1)
      ElseIf GetGadgetState(#Option_XML_CreateLoad_File_Private)
        XMLFileMethods::SetCreateLoadFile(2)
      EndIf
    Else
      XMLFileMethods::SetCreateLoadFile(0)
    EndIf 
    
    If GetGadgetState(#CheckBox_XML_Catch_File)
      If GetGadgetState(#Option_XML_Catch_File_Public)
        XMLFileMethods::SetCatchFile(1)
      ElseIf GetGadgetState(#Option_XML_Catch_File_Private)
        XMLFileMethods::SetCatchFile(2)
      EndIf
    Else
      XMLFileMethods::SetCatchFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_JSON_CreateLoad_File)
      If GetGadgetState(#Option_JSON_CreateLoad_File_Public)
        JSONFileMethods::SetCreateLoadFile(1)
      ElseIf GetGadgetState(#Option_JSON_CreateLoad_File_Private)
        JSONFileMethods::SetCreateLoadFile(2)
      EndIf
    Else
      JSONFileMethods::SetCreateLoadFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DataBase_CreateTable)
      If GetGadgetState(#Option_DataBase_CreateTable_Public)
        DataBaseMethods::SetCreateTable(1)
      ElseIf GetGadgetState(#Option_DataBase_CreateTable_Private)
        DataBaseMethods::SetCreateTable(2)
      EndIf
    Else
      DataBaseMethods::SetCreateTable(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DataBase_DropTable)
      If GetGadgetState(#Option_DataBase_DropTable_Public)
        DataBaseMethods::SetDropTable(1)
      ElseIf GetGadgetState(#Option_DataBase_DropTable_Private)
        DataBaseMethods::SetDropTable(2)
      EndIf
    Else
      DataBaseMethods::SetDropTable(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DataBase_TruncateTable)
      If GetGadgetState(#Option_DataBase_TruncateTable_Public)
        DataBaseMethods::SetTruncateTable(1)
      ElseIf GetGadgetState(#Option_DataBase_TruncateTable_Private)
        DataBaseMethods::SetTruncateTable(2)
      EndIf
    Else
      DataBaseMethods::SetTruncateTable(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DataBase_SelectTable)
      If GetGadgetState(#Option_DataBase_SelectTable_Public)
        DataBaseMethods::SetSelectTable(1)
      ElseIf GetGadgetState(#Option_DataBase_SelectTable_Private)
        DataBaseMethods::SetSelectTable(2)
      EndIf
    Else
      DataBaseMethods::SetSelectTable(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DataBase_CountRowInTable)
      If GetGadgetState(#Option_DataBase_CountRowInTable_Public)
        DataBaseMethods::SetCountLine(1)
      ElseIf GetGadgetState(#Option_DataBase_CountRowInTable_Private)
        DataBaseMethods::SetCountLine(2)
      EndIf
    Else
      DataBaseMethods::SetCountLine(0)
    EndIf 
    
    If GetGadgetState(#CheckBox_DataBase_OpenFile)
      If GetGadgetState(#Option_DataBase_OpenFile_Public)
        DataBaseMethods::SetOpenFile(1)
      ElseIf GetGadgetState(#Option_DataBase_OpenFile_Private)
        DataBaseMethods::SetOpenFile(2)
      EndIf
    Else
      DataBaseMethods::SetOpenFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DataBase_ModuleTo)
      If GetGadgetState(#Option_DataBase_ModuleTo_Public)
        DataBaseMethods::SetSendModuleTo(1)
      ElseIf GetGadgetState(#Option_DataBase_ModuleTo_Private)
        DataBaseMethods::SetSendModuleTo(2)
      EndIf
    Else
      DataBaseMethods::SetSendModuleTo(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DataBase_ToModule)
      If GetGadgetState(#Option_DataBase_ToModule_Public)
        DataBaseMethods::SetLoadModuleFrom(1)
      ElseIf GetGadgetState(#Option_DataBase_ToModule_Private)
        DataBaseMethods::SetLoadModuleFrom(2)
      EndIf
    Else
      DataBaseMethods::SetLoadModuleFrom(0)
    EndIf
    
    If GetGadgetState(#CheckBox_DataBase_Vacuumize)
      If GetGadgetState(#Option_DataBase_Vacuumize_Public)
        DataBaseMethods::SetVacummize(1)
      ElseIf GetGadgetState(#Option_DataBase_Vacuumize_Private)
        DataBaseMethods::SetVacummize(2)
      EndIf
    Else
      DataBaseMethods::SetVacummize(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Pref_Group)
      If GetGadgetState(#Option_Pref_Group_Public)
        PrefsFileMethods::SetReadWriteGroup(1)
      ElseIf GetGadgetState(#Option_Pref_Group_Private)
        PrefsFileMethods::SetReadWriteGroup(2)
      EndIf
    Else  
      PrefsFileMethods::SetReadWriteGroup(0)
    EndIf
    
    If GetGadgetState(#CheckBox_Pref_File)
      If GetGadgetState(#Option_Pref_File_Public)
        PrefsFileMethods::SetCreateOpenFile(1)
      ElseIf GetGadgetState(#Option_Pref_File_Private)
        PrefsFileMethods::SetCreateOpenFile(2)
      EndIf
    Else  
      PrefsFileMethods::SetCreateOpenFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_ReadWriteBinary)
      If GetGadgetState(#Option_ReadWriteBinary_Public)
        BinaryFileMethods::SetReadWrite(1)
      ElseIf GetGadgetState(#Option_ReadWriteBinary_Private)
        BinaryFileMethods::SetReadWrite(2)
      EndIf
    Else
      BinaryFileMethods::SetReadWrite(0)
    EndIf
    
    If GetGadgetState(#CheckBox_CreateBinary)
      If GetGadgetState(#Option_CreateBinary_Public)
        BinaryFileMethods::SetCreateFile(1)
      ElseIf GetGadgetState(#Option_CreateBinary_Private)
        BinaryFileMethods::SetCreateFile(2)
      EndIf
    Else
      BinaryFileMethods::SetCreateFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_OpenBinary)
      If GetGadgetState(#Option_OpenBinary_Public)
        BinaryFileMethods::SetOpenFile(1)
      ElseIf GetGadgetState(#Option_OpenBinary_Private)
        BinaryFileMethods::SetOpenFile(2)
      EndIf
    Else
      BinaryFileMethods::SetOpenFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_ReadBinary)
      If GetGadgetState(#Option_ReadBinary_Public)
        BinaryFileMethods::SetReadFile(1)
      ElseIf GetGadgetState(#Option_ReadBinary_Private)
        BinaryFileMethods::SetReadFile(2)
      EndIf
    Else
      BinaryFileMethods::SetReadFile(0)
    EndIf 
    
    If GetGadgetState(#CheckBox_ReadWriteEncoded)
      If GetGadgetState(#Option_ReadWriteEncoded_Public)
        EncodedFileMethods::SetReadWrite(1)
      ElseIf GetGadgetState(#Option_ReadWriteEncoded_Private)
        EncodedFileMethods::SetReadWrite(2)
      EndIf
    Else
      EncodedFileMethods::SetReadWrite(0)
    EndIf
    
    If GetGadgetState(#CheckBox_CreateEncoded)
      If GetGadgetState(#Option_CreateEncoded_Public)
        EncodedFileMethods::SetCreateFile(1)
      ElseIf GetGadgetState(#Option_CreateEncoded_Private)
        EncodedFileMethods::SetCreateFile(2)
      EndIf
    Else
      EncodedFileMethods::SetCreateFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_OpenEncoded)
      If GetGadgetState(#Option_OpenEncoded_Public)
        EncodedFileMethods::SetOpenFile(1)
      ElseIf GetGadgetState(#Option_OpenEncoded_Private)
        EncodedFileMethods::SetOpenFile(2)
      EndIf
    Else
      EncodedFileMethods::SetOpenFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_ReadEncoded)
      If GetGadgetState(#Option_ReadEncoded_Public)
        EncodedFileMethods::SetReadFile(1)
      ElseIf GetGadgetState(#Option_ReadEncoded_Private)
        EncodedFileMethods::SetReadFile(2)
      EndIf
    Else
      EncodedFileMethods::SetReadFile(0)
    EndIf
    
    EncodedFileMethods::SetExternalModuleToUse(GetGadgetText(#ComboBox_EncodedFile_Module)) 
    Field::CurrentFileModuleName(0, EncodedFileMethods::GetExternalModuleToUse())
    
    If GetGadgetState(#CheckBox_ReadWriteCrypted)
      If GetGadgetState(#Option_ReadWriteCrypted_Public)
        CryptedFileMethods::SetReadWrite(1)
      ElseIf GetGadgetState(#Option_ReadWriteCrypted_Private)
        CryptedFileMethods::SetReadWrite(2)
      EndIf
    Else
      CryptedFileMethods::SetReadWrite(0)
    EndIf
    
    If GetGadgetState(#CheckBox_CreateCrypted)
      If GetGadgetState(#Option_CreateCrypted_Public)
        CryptedFileMethods::SetCreateFile(1)
      ElseIf GetGadgetState(#Option_CreateCrypted_Private)
        CryptedFileMethods::SetCreateFile(2)
      EndIf
    Else
      CryptedFileMethods::SetCreateFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_OpenCrypted)
      If GetGadgetState(#Option_OpenCrypted_Public)
        CryptedFileMethods::SetOpenFile(1)
      ElseIf GetGadgetState(#Option_OpenCrypted_Private)
        CryptedFileMethods::SetOpenFile(2)
      EndIf
    Else
      CryptedFileMethods::SetOpenFile(0)
    EndIf
    
    If GetGadgetState(#CheckBox_ReadCrypted)
      If GetGadgetState(#Option_ReadCrypted_Public)
        CryptedFileMethods::SetReadFile(1)
      ElseIf GetGadgetState(#Option_ReadCrypted_Private)
        CryptedFileMethods::SetReadFile(2)
      EndIf
    Else
      CryptedFileMethods::SetReadFile(0)
    EndIf

    CryptedFileMethods::SetExternalModuleToUse(GetGadgetText(#ComboBox_CryptedFile_Module))
    Field::CurrentFileModuleName(1, CryptedFileMethods::GetExternalModuleToUse())
    
    If GetGadgetState(#CheckBox_Debugging)
      If GetGadgetState(#Option_Debugging_Public)
        DebuggingMethods::SetDebugging(1)
      ElseIf GetGadgetState(#Option_Debugging_Private)
        DebuggingMethods::SetDebugging(2)
      EndIf
    Else
      DebuggingMethods::SetDebugging(0)
    EndIf
    
    ProcedureReturn #True
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Nom de la procdure <<<<<
  
  Procedure To_GUI()

    SetGadgetText(#String_Module_Name, Field::GetModuleName())

    If Field::GetInstanceStyle() = Field::#INSTANCE_STYLE_UNIQUE
      SetGadgetState(#Option_Single_Instance, 1)
    ElseIf Field::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_ID
      SetGadgetState(#Option_Multiple_Instance_ID, 1)
    ElseIf Field::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_INDEX
      SetGadgetState(#Option_Multiple_Instance_Index, 1)
    ElseIf Field::GetInstanceStyle() = Field::#INSTANCE_STYLE_MULTIPLE_NAME
      SetGadgetState(#Option_Multiple_Instance_Name, 1)
    ElseIf Field::GetInstanceStyle() = Field::#INSTANCE_STYLE_OFF_MODULE  
      SetGadgetState(#Option_Off_Module_Instance, 1)
    EndIf
    
    If Field::FirstInstance(0)
      
      ClearGadgetItems(#Editor_Module_Primary)
      
      Repeat 
        AddGadgetItem(#Editor_Module_Primary, -1, Field::GetFieldDescription(0))
      Until Field::NextInstance(0) = #Null
      
    EndIf 
    
    If Field::FirstInstance(1)
      
      ClearGadgetItems(#Editor_Module_Secondary)
      
      Repeat 
        AddGadgetItem(#Editor_Module_Secondary, -1, Field::GetFieldDescription(1))
      Until Field::NextInstance(1) = #Null
      
    EndIf

    If Field::FirstInternalStruct()
      
      ClearGadgetItems(#ListView_Internal_Structure)
      
      Repeat
        AddGadgetItem(#ListView_Internal_Structure, -1, Field::GetStructName())
      Until Field::NextInternalStruct() = #Null
      
    EndIf
  
    MethodGadgetsStatus(BasicMethods::GetObservator(), #CheckBox_Observator)
    MethodGadgetsStatus(BasicMethods::GetMutator(), #CheckBox_Mutator)
    MethodGadgetsStatus(BasicMethods::GetUpdate(), #CheckBox_Update)
    MethodGadgetsStatus(BasicMethods::GetInitialize(), #CheckBox_Initialize)
    MethodGadgetsStatus(BasicMethods::GetReset(), #CheckBox_Reset)
    
    MethodGadgetsStatus(StaticArrayMethods::GetQuickSortAscendant(), #CheckBox_QuickSortAscending)
    MethodGadgetsStatus(StaticArrayMethods::GetQuickSortDescendant(), #CheckBox_QuickSortDescending)
    MethodGadgetsStatus(StaticArrayMethods::GetLinearSearch(), #CheckBox_LinearSearch)
    MethodGadgetsStatus(StaticArrayMethods::GetBinarySearch(), #CheckBox_BinarySearch)
    MethodGadgetsStatus(StaticArrayMethods::GetFixedLenghtQueue(), #CheckBox_FixedLengthQueue)
    
    MethodGadgetsStatus(LinkedListMethods::GetSort(), #CheckBox_Sort_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetLinearSearch(), #CheckBox_Lin_Search_Linked_List)
    MethodGadgetsStatus(LinkedListMethods::GetBinarySearch(), #CheckBox_Bin_Search_Linked_List)
    
    MethodGadgetsStatus(DynamicArrayMethods::GetSort(), #CheckBox_Sort_Dynamic_Array)
    MethodGadgetsStatus(DynamicArrayMethods::GetLinearSearch(), #CheckBox_Lin_Search_Dynamic_Array)
    MethodGadgetsStatus(DynamicArrayMethods::GetBinarySearch(), #CheckBox_Bin_Search_Dynamic_Array)
    
    MethodGadgetsStatus(MathsMethods::GetEqual(), #CheckBox_Equal)
    MethodGadgetsStatus(MathsMethods::GetPlus(), #CheckBox_Plus)
    MethodGadgetsStatus(MathsMethods::GetMinus(), #CheckBox_Minus)
    MethodGadgetsStatus(MathsMethods::GetProduct(), #CheckBox_Product)
    MethodGadgetsStatus(MathsMethods::GetDivide(), #CheckBox_Divide)
    MethodGadgetsStatus(MathsMethods::GetProductByScalar(), #CheckBox_ProductByScalar)
    MethodGadgetsStatus(MathsMethods::GetDivideByScalar(), #CheckBox_DivideByScalar)
    MethodGadgetsStatus(MathsMethods::GetCopy(), #CheckBox_Copy)
    MethodGadgetsStatus(MathsMethods::GetCompare(), #CheckBox_Compare)
    MethodGadgetsStatus(MathsMethods::GetSwapping(), #CheckBox_Swapping)
    
    MethodGadgetsStatus(XMLFileMethods::GetNodeWithChildNode(), #CheckBox_XML_SetGet_Node)
    MethodGadgetsStatus(XMLFileMethods::GetNodeWithAttributes(), #CheckBox_XML_SetGet_Attribute)
    MethodGadgetsStatus(XMLFileMethods::GetAddCollectAttributes(), #CheckBox_XML_AddCollect_Attribute)
    MethodGadgetsStatus(XMLFileMethods::GetCreateLoadFile(), #CheckBox_XML_CreateLoad_File)
    MethodGadgetsStatus(XMLFileMethods::GetCatchFile(), #CheckBox_XML_Catch_File)
    
    MethodGadgetsStatus(JSONFileMethods::GetCreateLoadFile(), #CheckBox_JSON_CreateLoad_File)
    
    MethodGadgetsStatus(DataBaseMethods::GetCreateTable(), #CheckBox_DataBase_CreateTable)
    MethodGadgetsStatus(DataBaseMethods::GetDropTable(), #CheckBox_DataBase_DropTable)
    MethodGadgetsStatus(DataBaseMethods::GetTruncateTable(), #CheckBox_DataBase_TruncateTable)
    MethodGadgetsStatus(DataBaseMethods::GetSelectTable(), #CheckBox_DataBase_SelectTable)
    MethodGadgetsStatus(DataBaseMethods::GetCountLine(), #CheckBox_DataBase_CountRowInTable)
    MethodGadgetsStatus(DataBaseMethods::GetOpenFile(), #CheckBox_DataBase_OpenFile)
    MethodGadgetsStatus(DataBaseMethods::GetSendModuleTo(), #CheckBox_DataBase_ModuleTo)
    MethodGadgetsStatus(DataBaseMethods::GetLoadModuleFrom(), #CheckBox_DataBase_ToModule)
    MethodGadgetsStatus(DataBaseMethods::GetVacummize(), #CheckBox_DataBase_Vacuumize)
    
    MethodGadgetsStatus(PrefsFileMethods::GetReadWriteGroup(), #CheckBox_Pref_Group)
    MethodGadgetsStatus(PrefsFileMethods::GetCreateOpenFile(), #CheckBox_Pref_File)
    
    MethodGadgetsStatus(BinaryFileMethods::GetReadWrite(), #CheckBox_ReadWriteBinary)
    MethodGadgetsStatus(BinaryFileMethods::GetCreateFile(), #CheckBox_CreateBinary)
    MethodGadgetsStatus(BinaryFileMethods::GetOpenFile(), #CheckBox_OpenBinary)
    MethodGadgetsStatus(BinaryFileMethods::GetReadFile(), #CheckBox_ReadBinary)
    
    MethodGadgetsStatus(EncodedFileMethods::GetReadWrite(), #CheckBox_ReadWriteEncoded)
    MethodGadgetsStatus(EncodedFileMethods::GetCreateFile(), #CheckBox_CreateEncoded)
    MethodGadgetsStatus(EncodedFileMethods::GetOpenFile(), #CheckBox_OpenEncoded)
    MethodGadgetsStatus(EncodedFileMethods::GetReadFile(), #CheckBox_ReadEncoded)
    SelectComboBoxGadgetItem(#ComboBox_EncodedFile_Module, EncodedFileMethods::GetExternalModuleToUse())
   
    MethodGadgetsStatus(CryptedFileMethods::GetReadWrite(), #CheckBox_ReadWriteCrypted)
    MethodGadgetsStatus(CryptedFileMethods::GetCreateFile(), #CheckBox_CreateCrypted)
    MethodGadgetsStatus(CryptedFileMethods::GetOpenFile(), #CheckBox_OpenCrypted)
    MethodGadgetsStatus(CryptedFileMethods::GetReadFile(), #CheckBox_ReadCrypted)
    SelectComboBoxGadgetItem(#ComboBox_CryptedFile_Module, CryptedFileMethods::GetExternalModuleToUse())
    
    MethodGadgetsStatus(DebuggingMethods::GetDebugging(), #CheckBox_Debugging)
    
    Field::Reset(Field::#RESET_STYLE_AFTER_RUN)
   
  EndProcedure
  
  Procedure SaveSourceCode(FileName.s)
    
    FileID = CreateFile(#PB_Any, FileName)
    
    If IsFile(FileID)
      
      WriteStringFormat(FileID, #PB_UTF8)
      
      Max = CountGadgetItems(#Editor_Source_Code) - 1
      
      For ItemID = 0 To Max
        WriteStringN(FileID, GetGadgetItemText(#Editor_Source_Code, ItemID))
      Next
      
      CloseFile(FileID)
      
      SetFileDate(Filename, #PB_Date_Created, Date())
      SetFileDate(Filename, #PB_Date_Modified, Date())
      SetFileDate(Filename, #PB_Date_Accessed, Date())
      
    EndIf
    
  EndProcedure
  
  Procedure Refresh()
    
    Field::Reset(Field::#RESET_STYLE_NEW_MODULE)
    
    BasicMethods::Reset()
    StaticArrayMethods::Reset()
    LinkedListMethods::Reset()
    DynamicArrayMethods::Reset()
    MathsMethods::Reset()
    XMLFileMethods::Reset()
    JSONFileMethods::Reset()
    DataBaseMethods::Reset()
    PrefsFileMethods::Reset()
    BinaryFileMethods::Reset()
    EncodedFileMethods::Reset()
    CryptedFileMethods::Reset()
    DebuggingMethods::Reset()
    
    ClearGadgetItems(#ListView_Internal_Structure)
    
    ClearGadgetItems(#Editor_Module_Primary)
    ClearGadgetItems(#Editor_Module_Secondary)
    ClearGadgetItems(#Editor_Source_Code)
    
    SetGadgetState(#Panel, 0)
    SetGadgetState(#Panel_Primary_Secondary, 0)
    
    To_GUI()
    
  EndProcedure
 
  Procedure SaveDevModuleProjectFile()
    
    FileName.s = SaveFileRequester(Language::Message("Select project to save"), Options::GetProjectSavePath(), Language::Message("DevModule Project (*.devm)|*.devm"), 1)
    
    If FileName <> ""
      
      From_GUI()
      
      FileName = CheckFileExtension::DoIt(FileName, "devm")
      
      If FileSize(FileName) <> -1
        
        If MessageRequester(Language::Message("Warning - File already exist"), ReplaceString(Language::Message("The file %FILE% already exist. Would you like to overwrite ?"), "%FILE%", Chr(34) + GetFilePart(FileName) + Chr(34)), #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
          
          If DeleteFile(FileName)
            CodeGenerator::SaveProjectFile(FileName)
          EndIf
          
        EndIf
        
      Else
        
        CodeGenerator::SaveProjectFile(FileName)
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure SaveDevModuleSourceCode()
    
    FileName.s = SaveFileRequester(Language::Message("Select the Source code to save"), Options::GetProjectSavePath(), Language::Message("PureBasic SourceCode (*.pb)|*.pb"), 1)
    
    If FileName <> ""
      
      FileName = CheckFileExtension::DoIt(FileName, "pb")
      
      If FileSize(FileName) <> -1
        
        If MessageRequester(Language::Message("Warning - File already exist"), ReplaceString(Language::Message("The file %FILE% already exist. Would you like to overwrite ?"), "%FILE%", Chr(34) + GetFilePart(FileName) + Chr(34)), #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
          
          If DeleteFile(FileName)
            SaveSourceCode(FileName)
          EndIf
          
        EndIf
        
      Else
        
        SaveSourceCode(FileName)
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure LoadDevModuleProjectFile()
    
    FileName.s = OpenFileRequester(Language::Message("Select project to open"), Options::GetProjectSavePath(), Language::Message("DevModule Project (*.devm)|*.devm"), 1)
    
    If FileName <> ""
      
      Field::Reset(Field::#RESET_STYLE_NEW_MODULE)
      
      CodeGenerator::LoadProjectFile(FileName)
      To_GUI()
      
      If GetGadgetState(#Option_Single_Instance) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable) 
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_ID) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_Index) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf
      
      If GetGadgetState(#Option_Multiple_Instance_Name) = 1
        DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      EndIf

      If GetGadgetState(#Option_Off_Module_Instance) = 1
        DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
        DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)    
      EndIf

      SetGadgetState(#Panel, 0)
      
    EndIf
    
  EndProcedure
  
  Procedure SaveFileDispatcher()

    Select GetGadgetState(#Panel)
    
    	Case 0, 1, 2, 3
    	  SaveDevModuleProjectFile()
    	  
    	Case 4
    	  SaveDevModuleSourceCode()
    	  
    EndSelect
    
  EndProcedure
  
  Procedure MatrixGenerator()
    
    If MatrixGenerator::Runner(#MainWin)
      
      SetGadgetState(#Option_Off_Module_Instance, 1)
      DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
      
      For Index = #CheckBox_Equal To #CheckBox_Swapping Step 3
        If Index <> #CheckBox_Divide
          SetGadgetState(Index, 1)
          DisableGadget(Index + 1, 0)
          DisableGadget(Index + 2, 0)
        EndIf
      Next
      
    EndIf
    
  EndProcedure
  
  Procedure MatrixEncodedGenerator()
    
    If MatrixEncodedGenerator::Runner(#MainWin)
      SetGadgetText(#Editor_Source_Code, CodeGenerator::Generate(1))
      SetGadgetState(#Panel, 4)
    EndIf
    
  EndProcedure
  
  Procedure Private_ScanInternalStructure(List FieldType.s(), EditorID)
    
    EditorMax = CountGadgetItems(EditorID)
    
    For ItemID = 0 To EditorMax
      
      Field.s = GetGadgetItemText(EditorID, ItemID)
      
      If FindString(Field, "::")
        FieldType.s = ""
      ElseIf FindString(Field, "(")
        FieldType = StringBetween::FindIt(Field, ".", "(")
      ElseIf FindString(Field, "[")
        FieldType = StringBetween::FindIt(Field, ".", "[")
      Else
        FieldType = StringField(Field, 2, ".")
      EndIf
      
      If Len(FieldType) > 1
        
        FoundAt.l = -1
        
        ForEach FieldType()
          If FieldType() = FieldType
            FoundAt = ListIndex(FieldType())
            Break
          EndIf
        Next

        If FoundAt = -1
          AddElement(FieldType())
          FieldType() = FieldType
        EndIf
        
      EndIf
      
    Next
    
  EndProcedure
  
  Procedure ScanInternalStructure()
    
    ClearGadgetItems(#ListView_Internal_Structure)
    Field::Reset(Field::#RESET_STYLE_FLUSH_INTERNAL_STRUCTURE)
    
    NewList FieldType.s()
    
    Private_ScanInternalStructure(FieldType(), #Editor_Module_Primary)
    Private_ScanInternalStructure(FieldType(), #Editor_Module_Secondary)
    
    ForEach FieldType()
      AddGadgetItem(#ListView_Internal_Structure, -1, FieldType()) 
      Field::AddInternalStruct(FieldType())
      FieldType() = ""
    Next
    
    FreeList(FieldType())
    
  EndProcedure
  
  Procedure InternalStructureEditor()
    
    StructName.s = GetGadgetText(#ListView_Internal_Structure)
    
    If Field::SearchInternalStruct(StructName) >= 0
      
      If Window::OpenEx(#InternalStructureWin, Language::Message("Internal Structure Editor"))
        
        TextGadget(#Text_Internal_Structure_Name, 5,5, Window::Width(#InternalStructureWin)-10, 25, Language::Message("Structure name") + " : " + StructName)
        EditorGadget(#Editor_Internal_Structure, 5, GadgetLocate::DownOf(#Text_Internal_Structure_Name, 5), Window::Width(#InternalStructureWin)-10, 325)
        
        Options::RefreshEditors(100, #Editor_Internal_Structure)
        Field::To_GUI(1)
        
        ButtonGroup::Horizontal(#Button_Internal_Structure_OK, #Button_Internal_Structure_CANCEL, 500-150*2-10, 400-35, 150, 30, 5)
        
        SetGadgetText(#Button_Internal_Structure_OK, Language::Message("OK"))
        SetGadgetText(#Button_Internal_Structure_CANCEL, Language::Message("Cancel"))
        
      EndIf
      
    EndIf
    
  EndProcedure
  
  Procedure CloseInternalStructureEditor(Mode.b)
    
    If Mode = 1
      PastedCodeCleanUp::DoIt(#Editor_Internal_Structure)
      Field::From_GUI(1)
    EndIf
    
    Window::GetCurrentPositionSize(#InternalStructureWin)
    CloseWindow(#InternalStructureWin)
    HideWindow(#MainWin, #False)
    
  EndProcedure
  
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< Nom de la procdure <<<<<
  
  Procedure GenerateCode()
    
    If From_GUI() = #True
      SetGadgetText(#Editor_Source_Code, CodeGenerator::Generate())
      SetGadgetState(#Panel, 4)
    EndIf
    
  EndProcedure
  
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; <<<<< Initialize (Public) <<<<<
  
  Procedure Initialize()
    
    Path.s = GetPathPart(ProgramFilename())
    SetCurrentDirectory(Path)
    
    CompilerSelect #PB_Compiler_OS
        
      CompilerCase #PB_OS_Windows 
        UserPath.s = GetEnvironmentVariable("APPDATA") + #PathSep + "DevModule" + #PathSep
        
        If Left(UserPath, 2) <> "C:"
          UserPath = "C:" + #PathSep + UserPath
        EndIf
        
      CompilerCase #PB_OS_Linux
        UserPath.s = GetHomeDirectory() + ".devmodule" + #PathSep
        
      CompilerCase #PB_OS_MacOS
        UserPath.s = GetHomeDirectory() + ".devmodule" + #PathSep ; This might be wrong
        
    CompilerEndSelect
    
    BuiltPathDirectory::DoIt(UserPath)
    LanguagePath.s = GetPathPart(ProgramFilename()) + "language" + #PathSep
    
    CompilerIf #PB_Compiler_Debugger
      BuiltLanguageDefaultFiles(LanguagePath)
      
      CompilerIf #PB_Compiler_OS = #PB_OS_Linux
        LinuxDesktopFile_N_Icon()
      CompilerEndIf 
      
    CompilerEndIf 
    
    PrefFileName = UserPath + "DevModule.prefs"
    
    Window::Initialize(#MainWin, 50, 50, 885, 595, #False)
    Window::Initialize(#InternalStructureWin, 50, 50, 500, 405, #False)

    If FindString(GetEnvironmentVariable("LANGUAGE"), "fr")
      Language::Initialize(LanguagePath, "Franais.xml", @Language_To_GUI()) 
    Else
      Language::Initialize(LanguagePath, "English.xml", @Language_To_GUI())
    EndIf
    
    Help::Initialize()
    Stats::Initialize()
    Options::Initialize()
    Comments::Initialize(#Version)
    AskPureBasicCompiler::Initialize()
    MatrixEncodedGenerator::Initialize()
    
    ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    ; <<<<< Chargement des icnes <<<<<
    
    UsePNGImageDecoder()
    
    Sticky::Initialize(#Button_PushPin, Path + "icons" + #PathSep,  "Push Pin 01 XP - 32.png", "Push Pin 02 XP - 32.png")
    LoadImage(#Button_New, Path + "icons" + #PathSep + "New XP - 32.png")
    LoadImage(#Button_Open, Path + "icons" + #PathSep + "Open XP - 32.png")
    LoadImage(#Button_Save, Path + "icons" + #PathSep + "Save XP - 32.png")
    LoadImage(#Button_Code, Path + "icons" + #PathSep + "Generate Code XP - 32.png")
    LoadImage(#Button_Brush, Path + "icons" + #PathSep + "Brush XP - 32.png")
    LoadImage(#Button_ClipBoard, Path + "icons" + #PathSep + "ClipBoard XP - 32.png")
    LoadImage(#Button_Language, Path + "icons" + #PathSep + "Language XP - 32.png")
    LoadImage(#Button_Quit, Path + "icons" + #PathSep + "Quit XP - 32.png")

    If FileSize(PrefFileName) = -1
      CreateDevModulePreferences()
    Else
      OpenDevModulePreferences()
    EndIf
    
    Language::LoadTableFile()
    Window::SanityCheck(#MainWin)
    Stats::Analysis()
    Help::SetPath(Language::GetFileName())
    
  EndProcedure
  
  Procedure CreateWindow()
    
    If Window::Open(#MainWin)
      
      CompilerSelect #PB_Compiler_OS
          
        CompilerCase #PB_OS_Windows 
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 25,"")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 805, 520)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              EditorGadget(#Editor_Module_Primary, 5, 5, 590, 480)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              EditorGadget(#Editor_Module_Secondary, 5, 5, 590, 480)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 445, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 375)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            
            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            
            FrameGadget(#Frame_Linked_List, 5, GadgetLocate::DownOf(#Frame_Static_Array), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            
            FrameGadget(#Frame_Dynamic_Array, 5, GadgetLocate::DownOf(#Frame_Linked_List), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 275, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 155, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 155, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Options
            
            BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650,60)
            
            FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 320, 150, "")
            TextGadget(#Text_Editor_FontName, 10, GadgetLocate::DownOf(#Frame_Project_SavePath, 35), 125, 25, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 25, "", #PB_String_ReadOnly)
            ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 25, "...")
            TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 25, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 25, "",  #PB_String_ReadOnly)
            TextGadget(#Text_Editor_FrontColor, 10, GadgetLocate::DownOf(#Text_Editor_FontSize, 5), 125, 25, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_FrontColor, GadgetLocate::RightOf(#Text_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 123, 20, Options::GetFrontColor())
            ButtonGadget(#Button_Editor_FrontColor, GadgetLocate::RightOf(#PreviewColor_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 50, 25, "...")
            TextGadget(#Text_Editor_BackColor, 10, GadgetLocate::DownOf(#Text_Editor_FrontColor, 5), 125, 25, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_BackColor, GadgetLocate::RightOf(#Text_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 123, 20, Options::GetBackColor())
            ButtonGadget(#Button_Editor_BackColor, GadgetLocate::RightOf(#PreviewColor_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 50, 25, "...")
            
            MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), #Frame_User, #Text_User_Email, 320)
            
            FrameGadget(#Frame_Source_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 115, "")
            OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, "")
            OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, "")
            TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
            
            DateFormatGadgetGroup(#Frame_Date_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 325, 55)
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            EditorGadget(#Editor_Source_Code, 5, 5, 800, 545)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 805, 25)
            WebGadget(#Web_Html_Help_File, 5, 35, 805, 515, "")
            
          CloseGadgetList()

        CompilerCase #PB_OS_Linux
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 35, "")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 810, 495)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              EditorGadget(#Editor_Module_Primary, 5, 5, 590, 445)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              EditorGadget(#Editor_Module_Secondary, 5, 5, 590, 445)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 445, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 375)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            
            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            
            FrameGadget(#Frame_Linked_List, 5, GadgetLocate::DownOf(#Frame_Static_Array), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            
            FrameGadget(#Frame_Dynamic_Array, 5, GadgetLocate::DownOf(#Frame_Linked_List), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 275, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Options
            
            BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650)
            
            FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 320, 190, "")
            TextGadget(#Text_Editor_FontName, 10, GadgetLocate::DownOf(#Frame_Project_SavePath, 35), 125, 34, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 34, "", #PB_String_ReadOnly)
            ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 34, "...")
            TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 34, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 34, "",  #PB_String_ReadOnly)
            TextGadget(#Text_Editor_FrontColor, 10, GadgetLocate::DownOf(#Text_Editor_FontSize, 5), 125, 34, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_FrontColor, GadgetLocate::RightOf(#Text_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 123, 32, Options::GetFrontColor())
            ButtonGadget(#Button_Editor_FrontColor, GadgetLocate::RightOf(#PreviewColor_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 50, 34, "...")
            TextGadget(#Text_Editor_BackColor, 10, GadgetLocate::DownOf(#Text_Editor_FrontColor, 5), 125, 34, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_BackColor, GadgetLocate::RightOf(#Text_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 123, 32, Options::GetBackColor())
            ButtonGadget(#Button_Editor_BackColor, GadgetLocate::RightOf(#PreviewColor_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 50, 34, "...")
            
            MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), #Frame_User, #Text_User_Email, 320)
            
            FrameGadget(#Frame_Source_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 115, "")
            OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, "")
            OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, "")
            TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
            
            DateFormatGadgetGroup(#Frame_Date_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 325)
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            EditorGadget(#Editor_Source_Code, 5, 5, 810, 535)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 810, 30)
            WebGadget(#Web_Html_Help_File, 5, 40, 810, 505, "")
            
          CloseGadgetList()
          
        CompilerCase #PB_OS_MacOS ; Maybe the interface need some adjustment 
          
          PanelGadget(#Panel, 5, 5, Window::Width(#MainWin) - 65, Window::Height(#MainWin) - 10)
            
            AddGadgetItem(#Panel, 0, "") ; Dfinition du module
            
            TextGadget(#Text_Module_Name, 5, 7, 150, 20, "")
            StringGadget(#String_Module_Name, GadgetLocate::RightOf(#Text_Module_Name, 5), 5, 350, 35,"")
            
            PanelGadget(#Panel_Primary_Secondary, 5, GadgetLocate::DownOf(#String_Module_Name, 5), 810, 495)
              
              AddGadgetItem(#Panel_Primary_Secondary, 0, "")
              EditorGadget(#Editor_Module_Primary, 5, 5, 590, 445)
              OptionGroup::Vertical(#Option_Single_Instance, #Option_Off_Module_Instance, GadgetLocate::RightOf(#Editor_Module_Primary, 5), 5, 210, 30)
              
              AddGadgetItem(#Panel_Primary_Secondary, 1, "") 
              EditorGadget(#Editor_Module_Secondary, 5, 5, 590, 445)
              
              AddGadgetItem(#Panel_Primary_Secondary, 2, "") ; Dfinition des structures Internes
              
              FrameGadget(#Frame_Internal_Structure, 5, 5, 370, 445, "")
              ButtonGadget(#Button_Scan_Internal_Structure, 10, 30, 360, 30, "")
              ListViewGadget(#ListView_Internal_Structure, 10, GadgetLocate::DownOf(#Button_Scan_Internal_Structure, 5), 360, 375)
              
            CloseGadgetList()
            
            AddGadgetItem(#Panel, 1, "") ; Slection des mthodes Partie 1
            
            FrameGadget(#Frame_Basic_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_Observator, GadgetX(#Frame_Basic_Methods) + 5, GadgetY(#Frame_Basic_Methods) + 20)
            MethodGadgets(#CheckBox_Mutator, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Observator))
            MethodGadgets(#CheckBox_Update, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Mutator))
            MethodGadgets(#CheckBox_Initialize, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Update))
            MethodGadgets(#CheckBox_Reset, GadgetX(#Frame_Basic_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Initialize))
            
            FrameGadget(#Frame_Static_Array, 5, GadgetLocate::DownOf(#Frame_Basic_Methods), 400, 150, "")
            MethodGadgets(#CheckBox_QuickSortAscending, GadgetX(#Frame_Static_Array) + 5, GadgetY(#Frame_Static_Array) + 20)
            MethodGadgets(#CheckBox_QuickSortDescending, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortAscending))
            MethodGadgets(#CheckBox_LinearSearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_QuickSortDescending))
            MethodGadgets(#CheckBox_BinarySearch, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_LinearSearch))
            MethodGadgets(#CheckBox_FixedLengthQueue, GadgetX(#Frame_Static_Array) + 5, GadgetLocate::DownOf(#CheckBox_BinarySearch))
            
            FrameGadget(#Frame_Linked_List, 5, GadgetLocate::DownOf(#Frame_Static_Array), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetY(#Frame_Linked_List) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Linked_List))
            MethodGadgets(#CheckBox_Bin_Search_Linked_List, GadgetX(#Frame_Linked_List) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Linked_List))
            
            FrameGadget(#Frame_Dynamic_Array, 5, GadgetLocate::DownOf(#Frame_Linked_List), 400, 100, "")
            MethodGadgets(#CheckBox_Sort_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetY(#Frame_Dynamic_Array) + 20)
            MethodGadgets(#CheckBox_Lin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Sort_Dynamic_Array))
            MethodGadgets(#CheckBox_Bin_Search_Dynamic_Array, GadgetX(#Frame_Dynamic_Array) + 5, GadgetLocate::DownOf(#CheckBox_Lin_Search_Dynamic_Array))
            
            FrameGadget(#Frame_Maths_Operator, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), 5, 400, 275, "")
            MethodGadgets(#CheckBox_Equal, GadgetX(#Frame_Maths_Operator) + 5, GadgetY(#Frame_Maths_Operator) + 20)
            MethodGadgets(#CheckBox_Plus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Equal))
            MethodGadgets(#CheckBox_Minus, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Plus))
            MethodGadgets(#CheckBox_Product, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Minus))
            MethodGadgets(#CheckBox_Divide, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Product))
            MethodGadgets(#CheckBox_ProductByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Divide))
            MethodGadgets(#CheckBox_DivideByScalar, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_ProductByScalar))
            
            MethodGadgets(#CheckBox_Copy, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_DivideByScalar))
            MethodGadgets(#CheckBox_Compare, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Copy))
            MethodGadgets(#CheckBox_Swapping, GadgetX(#Frame_Maths_Operator) + 5, GadgetLocate::DownOf(#CheckBox_Compare))
            
            FrameGadget(#Frame_Debugging, GadgetLocate::RightOf(#Frame_Basic_Methods, 5), GadgetLocate::DownOf(#Frame_Maths_Operator), 400, 50, "")
            MethodGadgets(#CheckBox_Debugging, GadgetX(#Frame_Debugging) + 5, GadgetY(#Frame_Debugging) + 20)
            
            AddGadgetItem(#Panel, 2, "") ; Slection des mthodes Partie 2
            
            FrameGadget(#Frame_XMLFile_Methods, 5, 5, 400, 150, "")
            MethodGadgets(#CheckBox_XML_SetGet_Node, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetY(#Frame_XMLFile_Methods) + 20)
            MethodGadgets(#CheckBox_XML_SetGet_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Node))
            MethodGadgets(#CheckBox_XML_AddCollect_Attribute, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_SetGet_Attribute))
            MethodGadgets(#CheckBox_XML_CreateLoad_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_AddCollect_Attribute))
            MethodGadgets(#CheckBox_XML_Catch_File, GadgetX(#Frame_XMLFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_XML_CreateLoad_File))
            
            FrameGadget(#Frame_JSONFile_Methods, 5, GadgetLocate::DownOf(#Frame_XMLFile_Methods), 400, 50, "")
            MethodGadgets(#CheckBox_JSON_CreateLoad_File, GadgetX(#Frame_JSONFile_Methods) + 5, GadgetY(#Frame_JSONFile_Methods) + 20)
            
            FrameGadget(#Frame_DataBaseFile_Methods, 5, GadgetLocate::DownOf(#Frame_JSONFile_Methods), 400, 250, "")
            MethodGadgets(#CheckBox_DataBase_CreateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetY(#Frame_DataBaseFile_Methods) + 20)
            MethodGadgets(#CheckBox_DataBase_DropTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CreateTable))
            MethodGadgets(#CheckBox_DataBase_TruncateTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_DropTable))
            MethodGadgets(#CheckBox_DataBase_SelectTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_TruncateTable))
            MethodGadgets(#CheckBox_DataBase_CountRowInTable, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_SelectTable))
            MethodGadgets(#CheckBox_DataBase_OpenFile, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_CountRowInTable))
            MethodGadgets(#CheckBox_DataBase_ModuleTo, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_OpenFile))
            MethodGadgets(#CheckBox_DataBase_ToModule, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ModuleTo))
            MethodGadgets(#CheckBox_DataBase_Vacuumize, GadgetX(#Frame_DataBaseFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_DataBase_ToModule))
            
            FrameGadget(#Frame_Preference_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), 5, 400, 75, "")
            MethodGadgets(#CheckBox_Pref_Group, GadgetX(#Frame_Preference_Methods) + 5, GadgetY(#Frame_Preference_Methods) + 20)
            MethodGadgets(#CheckBox_Pref_File, GadgetX(#Frame_Preference_Methods) + 5, GadgetLocate::DownOf(#CheckBox_Pref_Group))
            
            FrameGadget(#Frame_BinaryFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_Preference_Methods), 400, 125, "")
            MethodGadgets(#CheckBox_ReadWriteBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetY(#Frame_BinaryFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteBinary))
            MethodGadgets(#CheckBox_OpenBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateBinary))
            MethodGadgets(#CheckBox_ReadBinary, GadgetX(#Frame_BinaryFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenBinary))
            
            FrameGadget(#Frame_EncodedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_BinaryFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetY(#Frame_EncodedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteEncoded))
            MethodGadgets(#CheckBox_OpenEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateEncoded))
            MethodGadgets(#CheckBox_ReadEncoded, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenEncoded))
            ExternalModuleGadgetGroup(#ComboBox_EncodedFile_Module, GadgetX(#Frame_EncodedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadEncoded), 365)
            
            FrameGadget(#Frame_CryptedFile_Methods, GadgetLocate::RightOf(#Frame_XMLFile_Methods, 5), GadgetLocate::DownOf(#Frame_EncodedFile_Methods), 400, 165, "")
            MethodGadgets(#CheckBox_ReadWriteCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetY(#Frame_CryptedFile_Methods) + 20)
            MethodGadgets(#CheckBox_CreateCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadWriteCrypted))
            MethodGadgets(#CheckBox_OpenCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_CreateCrypted))
            MethodGadgets(#CheckBox_ReadCrypted, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_OpenCrypted))
            ExternalModuleGadgetGroup(#ComboBox_CryptedFile_Module, GadgetX(#Frame_CryptedFile_Methods) + 5, GadgetLocate::DownOf(#CheckBox_ReadCrypted), 365)
            
            AddGadgetItem(#Panel, 3, "") ; Options
            
            BrowsePathGadgetGroup(#Frame_Project_SavePath, 5, 5, 650)
            
            FrameGadget(#Frame_Editor_Color, 5, GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 320, 190, "")
            TextGadget(#Text_Editor_FontName, 10, GadgetLocate::DownOf(#Frame_Project_SavePath, 35), 125, 34, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontName, GadgetLocate::RightOf(#Text_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 125, 34, "", #PB_String_ReadOnly)
            ButtonGadget(#Button_Editor_FontSelect, GadgetLocate::RightOf(#String_Editor_FontName, 5), GadgetY(#Text_Editor_FontName), 50, 34, "...")
            TextGadget(#Text_Editor_FontSize, 10, GadgetLocate::DownOf(#Text_Editor_FontName, 5), 125, 34, "", #PB_Text_Center)
            StringGadget(#String_Editor_FontSize, GadgetLocate::RightOf(#Text_Editor_FontSize, 5), GadgetY(#Text_Editor_FontSize), 125, 34, "",  #PB_String_ReadOnly)
            TextGadget(#Text_Editor_FrontColor, 10, GadgetLocate::DownOf(#Text_Editor_FontSize, 5), 125, 34, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_FrontColor, GadgetLocate::RightOf(#Text_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 123, 32, Options::GetFrontColor())
            ButtonGadget(#Button_Editor_FrontColor, GadgetLocate::RightOf(#PreviewColor_Editor_FrontColor, 5), GadgetY(#Text_Editor_FrontColor), 50, 34, "...")
            TextGadget(#Text_Editor_BackColor, 10, GadgetLocate::DownOf(#Text_Editor_FrontColor, 5), 125, 34, "", #PB_Text_Center)
            ColorPreview::Gadget(#PreviewColor_Editor_BackColor, GadgetLocate::RightOf(#Text_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 123, 32, Options::GetBackColor())
            ButtonGadget(#Button_Editor_BackColor, GadgetLocate::RightOf(#PreviewColor_Editor_BackColor, 5), GadgetY(#Text_Editor_BackColor), 50, 34, "...")
            
            MethodsGadgetTextStringGroup(5, GadgetLocate::DownOf(#Frame_Editor_Color, 5), #Frame_User, #Text_User_Email, 320)
            
            FrameGadget(#Frame_Source_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Project_SavePath, 5), 325, 115, "")
            OptionGadget(#Option_Use_NBSPACE_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetY(#Frame_Source_Formatting) + 20, 295, 20, "")
            OptionGadget(#Option_Use_ASCII_Tabulation, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 40), 295, 20, "")
            TrackBarGadgetGroup(#TrackBar_Tabulation_Size, #Text_Tabulation_Size, GadgetX(#Frame_Source_Formatting) + 10, GadgetLocate::DownOf(#Option_Use_NBSPACE_Tabulation, 5), 305)
            
            DateFormatGadgetGroup(#Frame_Date_Formatting, GadgetLocate::RightOf(#Frame_Editor_Color, 5), GadgetLocate::DownOf(#Frame_Source_Formatting, 5), 325)
            
            AddGadgetItem(#Panel, 4, "") ; Code source
            EditorGadget(#Editor_Source_Code, 5, 5, 810, 535)
            
            AddGadgetItem(#Panel, 5, "")
            ComboBoxGadget(#ComboBox_Html_Help_Files, 5, 5, 810, 30)
            WebGadget(#Web_Html_Help_File, 5, 40, 810, 505, "")
            
          CloseGadgetList()
         
      CompilerEndSelect
      
      ButtonImageGroup::Vertical(#Button_PushPin, #Button_Quit, GadgetLocate::RightOf(#Panel, 5), 5, 50, 50, 2)
      
      DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
      DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
      
      Help::SetComboBoxGadgetID(#ComboBox_Html_Help_Files)
      Help::SetWebGadgetID(#Web_Html_Help_File)
      
      Options::GadgetLinking(#String_Project_SavePath, 0)
      Options::GadgetLinking(#String_Editor_FontName, 1)
      Options::GadgetLinking(#String_Editor_FontSize, 2)
      Options::GadgetLinking(#PreviewColor_Editor_FrontColor, 3)
      Options::GadgetLinking(#PreviewColor_Editor_BackColor, 4)
      Options::GadgetLinking(#Editor_Module_Primary, 5)
      Options::GadgetLinking(#Editor_Module_Secondary, 6)
      Options::GadgetLinking(#Editor_Source_Code, 7)

      Comments::GadgetLinking(#String_User_Name, 0)
      Comments::GadgetLinking(#String_User_NickName, 1)
      Comments::GadgetLinking(#String_User_Email, 2)
      Comments::GadgetLinking(#Option_Use_NBSPACE_Tabulation, 3)
      Comments::GadgetLinking(#TrackBar_Tabulation_Size, 4)
      Comments::GadgetLinking(#ComboBox_Date_Format, 5)

      MatrixGenerator::GadgetLinking(#String_Module_Name, 0)
      MatrixGenerator::GadgetLinking(#Editor_Module_Primary, 1)
      
      Field::GadgetLinking(#ComboBox_EncodedFile_Module, 0)
      Field::GadgetLinking(#ComboBox_CryptedFile_Module, 1)
      Field::GadgetLinking(#ListView_Internal_Structure, 2)
      Field::GadgetLinking(#Editor_Internal_Structure, 3)
      
      BindGadgetEvent(#Option_Use_NBSPACE_Tabulation, @TrackBarFollowOption())
      BindGadgetEvent(#Option_Use_ASCII_Tabulation, @TrackBarFollowOption())
      
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Alt | #PB_Shortcut_F4, #Button_Quit)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F5, #Button_Code) 
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F6, #Button_ClipBoard) 
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_F7, #Button_Brush)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F1, #Shortcut_Matrix_Field_Setup)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F2, #ShortCut_Matrix_Encoded)
      AddKeyboardShortcut(#MainWin, #PB_Shortcut_Control | #PB_Shortcut_F3, #ShortCut_Stats_Windows)
      
      Sticky::ToggleStatus(#MainWin, #True)
      Language::To_GUI()
      Options::To_GUI()
      Field::FileModule_To_GUI()
      Comments::To_GUI()
      Help::ScanHelpFolder()
      To_GUI()
      
    Else
      
      ShutDown()
      
    EndIf

  EndProcedure
  
  Procedure EventManager()

    Repeat
      
      Select WaitWindowEvent()
          
        Case #PB_Event_CloseWindow
          
          Select EventWindow()
              
            Case #InternalStructureWin
              CloseInternalStructureEditor(0)
              
            Case #MainWin
              Shutdown()
              
          EndSelect
          
        Case #PB_Event_Menu
          
          Select EventMenu()
              
            Case #Button_ClipBoard
              SetClipboardText(GetGadgetText(#Editor_Source_Code))
              
            Case #Button_Code
              CodeGenerator::Generate()
              
            Case #Button_Brush
              PastedCodeCleanUp::DoIt(#Editor_Module_Primary)
              PastedCodeCleanUp::DoIt(#Editor_Module_Secondary)
              
            Case #Shortcut_Matrix_Field_Setup
              MatrixGenerator()
              
            Case #ShortCut_Matrix_Encoded
              MatrixEncodedGenerator()
              
            Case #ShortCut_Stats_Windows
              Stats::OpenStatsWindow(#MainWin)
                
          EndSelect
          
        Case #PB_Event_Gadget
          
          Select EventGadget()
              
            Case #Button_Internal_Structure_OK
              CloseInternalStructureEditor(1)
 
            Case #Button_Internal_Structure_CANCEL
              CloseInternalStructureEditor(0)
              
            Case #Button_Editor_FrontColor
              Options::AskNewColor(0)

            Case #Button_Editor_BackColor
              Options::AskNewColor(1)

            Case #Button_Editor_FontSelect
              Options::AskNewFont()
              
            Case #Button_Browse_Project_SavePath
              Options::AskSavePath()
              
            Case #Button_Delete_Date_Mask
              Comments::DeleteSelectedDateMask()
              
            Case #Button_Date_Format_Editor
              Comments::AskDateMaskEditor()
              
            Case #Button_PushPin
              Sticky::ToggleStatus(#MainWin)
              
            Case #Button_New
              Refresh()
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Button_Open
              LoadDevModuleProjectFile()
              
            Case #Button_Save
              SaveFileDispatcher()
              
            Case #Button_Code
              GenerateCode()
                    
            Case #Button_Brush
              PastedCodeCleanUp::DoIt(#Editor_Module_Primary)
              PastedCodeCleanUp::DoIt(#Editor_Module_Secondary)
              
            Case #Button_Scan_Internal_Structure
              ScanInternalStructure()
              
            Case #ListView_Internal_Structure
              If EventType() = #PB_EventType_LeftDoubleClick
                If GetGadgetState(#ListView_Internal_Structure) >= 0
                  HideWindow(#MainWin, #True)
                  InternalStructureEditor()
                EndIf
              EndIf
              
            Case #Button_Add_EncodedFile_Module
              Field::AddFileModule(0)
              
            Case #Button_Add_CryptedFile_Module
              Field::AddFileModule(1)
              
            Case #Button_Remove_EncodedFile_Module
              Field::RemoveFileModule(0)
              
            Case #Button_Remove_CryptedFile_Module
              Field::RemoveFileModule(1)
              
            Case #Button_ClipBoard
              SetClipboardText(GetGadgetText(#Editor_Source_Code))
              
            Case #Button_Language
              Language::Requester()
              Help::SetPath(Language::GetFileName())
              Help::ScanHelpFolder()
              
            Case #Button_Quit
              Shutdown()
 
            Case #Option_Single_Instance
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_CountRowInTable, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_OpenFile, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_ID
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_Index
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Multiple_Instance_Name
              DependentGadgets::DoIt(0, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(1, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)

            Case #Option_Off_Module_Instance
              DependentGadgets::DoIt(1, #CheckBox_Equal, #CheckBox_Swapping, #CheckBox_Plus - #CheckBox_Equal)
              DependentGadgets::DoIt(0, #CheckBox_DataBase_CreateTable, #CheckBox_DataBase_Vacuumize, #CheckBox_DataBase_DropTable - #CheckBox_DataBase_CreateTable)
              
            Case #ComboBox_Html_Help_Files
              Help::DirectlySelectHtmlFiles()
              
          EndSelect
          
      EndSelect
      
    ForEver
    
  EndProcedure
  
EndModule

; <<<<<<<<<<<<<<<<<<<<<<<<<<
; <<<<< FIN DU FICHIER <<<<<
; <<<<<<<<<<<<<<<<<<<<<<<<<<
; IDE Options = PureBasic 5.70 beta 1 (Linux - x64)
; Folding = jAE7+
; EnableXP
; CompileSourceDirectory